import click
from extractor import extract_data_from_url, extract_from_enc_data
from qrreader import get_url_from_image


@click.command()
@click.option("-f","--file", is_flag=True, default=False)
@click.argument("string", type=str, required=True)
def get_google_auth_keys(file, string):
    '''STRING should be the filename or the otpauth url'''
    print(string)
    url = get_url_from_image(string) if file else string
    print(url)
    data = extract_data_from_url(url)
    print(data)
    extract_from_enc_data(data)


if __name__ == "__main__":
    get_google_auth_keys()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/protobuff/gauth.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19src/protobuff/gauth.proto\"\x8c\x05\n\x10MigrationPayload\x12\x37\n\x0eotp_parameters\x18\x01 \x03(\x0b\x32\x1f.MigrationPayload.OtpParameters\x12\x0f\n\x07version\x18\x02 \x01(\x05\x12\x12\n\nbatch_size\x18\x03 \x01(\x05\x12\x13\n\x0b\x62\x61tch_index\x18\x04 \x01(\x05\x12\x10\n\x08\x62\x61tch_id\x18\x05 \x01(\x05\x1a\xd5\x01\n\rOtpParameters\x12\x0e\n\x06secret\x18\x01 \x01(\x0c\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x0e\n\x06issuer\x18\x03 \x01(\t\x12.\n\talgorithm\x18\x04 \x01(\x0e\x32\x1b.MigrationPayload.Algorithm\x12,\n\x06\x64igits\x18\x05 \x01(\x0e\x32\x1c.MigrationPayload.DigitCount\x12\'\n\x04type\x18\x06 \x01(\x0e\x32\x19.MigrationPayload.OtpType\x12\x0f\n\x07\x63ounter\x18\x07 \x01(\x03\"y\n\tAlgorithm\x12\x19\n\x15\x41LGORITHM_UNSPECIFIED\x10\x00\x12\x12\n\x0e\x41LGORITHM_SHA1\x10\x01\x12\x14\n\x10\x41LGORITHM_SHA256\x10\x02\x12\x14\n\x10\x41LGORITHM_SHA512\x10\x03\x12\x11\n\rALGORITHM_MD5\x10\x04\"U\n\nDigitCount\x12\x1b\n\x17\x44IGIT_COUNT_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x44IGIT_COUNT_SIX\x10\x01\x12\x15\n\x11\x44IGIT_COUNT_EIGHT\x10\x02\"I\n\x07OtpType\x12\x18\n\x14OTP_TYPE_UNSPECIFIED\x10\x00\x12\x11\n\rOTP_TYPE_HOTP\x10\x01\x12\x11\n\rOTP_TYPE_TOTP\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'src.protobuff.gauth_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_MIGRATIONPAYLOAD']._serialized_start=30
  _globals['_MIGRATIONPAYLOAD']._serialized_end=682
  _globals['_MIGRATIONPAYLOAD_OTPPARAMETERS']._serialized_start=184
  _globals['_MIGRATIONPAYLOAD_OTPPARAMETERS']._serialized_end=397
  _globals['_MIGRATIONPAYLOAD_ALGORITHM']._serialized_start=399
  _globals['_MIGRATIONPAYLOAD_ALGORITHM']._serialized_end=520
  _globals['_MIGRATIONPAYLOAD_DIGITCOUNT']._serialized_start=522
  _globals['_MIGRATIONPAYLOAD_DIGITCOUNT']._serialized_end=607
  _globals['_MIGRATIONPAYLOAD_OTPTYPE']._serialized_start=609
  _globals['_MIGRATIONPAYLOAD_OTPTYPE']._serialized_end=682
# @@protoc_insertion_point(module_scope)



from PIL import Image
from pyzbar.pyzbar import decode
from loguru import logger


def get_url_from_image(image):
    logger.debug(f'Getting data from QRCODE in {image}')
    img = Image.open(image)
    decoded_lst = decode(img)
    logger.debug(f'Found {len(decoded_lst)} urls')
    return decoded_lst[0].data if decoded_lst[0].type == 'QRCODE' else None


if __name__ == "__main__":
    get_url_from_image('IMG_5971.PNG')


import base64
from loguru import logger
import protobuff.gauth_pb2 as pb
from urllib import parse
from google.protobuf.json_format import MessageToDict
from rich import print

def extract_from_enc_data(encoded_data):
    logger.debug(f"Extracting from {encoded_data}")
    data = base64.b64decode(encoded_data.replace(' ', '+'), validate=True)
    logger.debug(f"base 64 decoded: {data}")
    message = pb.MigrationPayload()
    logger.debug(f"Parsing from data")
    message.ParseFromString(data)
    logger.debug(f"extracted {message.otp_parameters}")
    data_list = MessageToDict(message)['otpParameters']
    for i, j in enumerate(data_list):
        j['secret'] = str(base64.b32encode(message.otp_parameters[i].secret), 'utf-8')
    logger.debug(f"Extracted dictionaries: {data_list}")
    print(data_list)

def extract_data_from_url(url):
    logger.debug(f"Extracting data from {url}")
    p = parse.urlparse(url)
    #TODO: try/cath
    return parse.parse_qs(str(p.query,'utf-8'))['data'][0]


if __name__ == "__main__":
    logger.debug(f"Starting Job")
    data = extract_data_from_url(b'otpauth-migration://offline?data=CicKCvh1mUhv1cwxYNESBHJvb3QaDTE5Mi4xNjguMTAwLjEgASgBMAIQAhgB')
    extract_from_enc_data(data)